package com.gic.haoban.manage.service.service.impl;

import com.gic.api.base.commons.Page;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.HandoverExternalDTO;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.dao.mapper.*;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.HandoverService;
import com.gic.haoban.manage.service.service.SecretSettingService;
import com.gic.wechat.api.dto.qywx.QywxUnassignedInfoDTO;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.slf4j.LoggerFactory.getLogger;

/**
 * Created 2021/6/15.
 *
 * @author hua
 */
@Service
public class HandoverServiceImpl implements HandoverService {

    private static final Logger logger = getLogger(HandoverServiceImpl.class);

    @Autowired
    private HandoverStaffMapper handoverStaffMapper;

    @Autowired
    private HandoverExternalMapper handoverExternalMapper;


    @Autowired
    private HandoverTransferMapper handoverTransferMapper;

    @Autowired
    private StaffMapper staffMapper;

    @Autowired
    private TabHaobanExternalClerkRelatedMapper externalClerkRelatedMapper;

    @Autowired
    private QywxUserApiService qywxUserApiService;

    @Autowired
    private Config config;

    @Autowired
    private WxEnterpriseMapper wxEnterpriseMapper;

    @Autowired
    private SecretSettingService secretSettingService;

    @Autowired
    private TabHaobanClerkMainStoreRelatedMapper clerkMainStoreRelatedMapper;

    @Override
    public boolean saveHandover(String wxEnterpriseId, List<QywxUnassignedInfoDTO> infoList) {
        TabHaobanWxEnterprise wxEnterprise = wxEnterpriseMapper.selectByPrimaryKey(wxEnterpriseId);
        if (CollectionUtils.isEmpty(infoList)) {
            logger.info("保存失败");
            return false;
        }
        Map<String, List<QywxUnassignedInfoDTO>> handoverUserMap = infoList.stream().collect(Collectors.groupingBy(dto -> dto.getHandoverUserid()));
        handoverUserMap.forEach((userId, list) -> {
            TabHandoverStaff handoverStaff = getHandoverStaff(wxEnterpriseId, userId);
            if (null == handoverStaff) {
                logger.info("staff已经存在不许重复");
                return;
            }
            List<TabHandoverExternal> externalList = list.stream().map(dto -> {
                TabHandoverExternal mid = getHandExternal(dto, wxEnterprise);
                mid.setStaffId(handoverStaff.getStaffId());
                handoverStaff.setDimissionTime(dto.getDimissionTimeDate());
                mid.setStatusFlag(mid.getStatusFlag() != null ? mid.getStatusFlag() : 1);
                mid.setUserId(dto.getHandoverUserid());
                return mid;
            }).collect(Collectors.toList());
            //个数
            int sum = externalList.stream().filter(dto -> dto.getHaobanFriendFlag() == 1).mapToInt(dto -> 1).sum();
            handoverStaff.setFriendCount(externalList.size());
            handoverStaff.setHaobanFriendCount(sum);
            //保存
            handoverStaffMapper.insertSelective(handoverStaff);
            handoverExternalMapper.insertBatch(externalList);

        });
        return false;
    }

    private TabHandoverStaff getHandoverStaff(String wxEnterpriseId, String wxUserId) {
        TabHaobanStaff haobanStaff = staffMapper.getNoStatusByWxUserIds(wxEnterpriseId, wxUserId);
        if (null == haobanStaff) {
            logger.info("对应的staff不存在：{}", wxUserId);
            return null;
        }
        TabHandoverStaff handoverStaff = handoverStaffMapper.getByStaffId(wxEnterpriseId, haobanStaff.getStaffId());
        if (null != handoverStaff) {
            logger.info("对应的handoverStaff不存在：{}", haobanStaff.getStaffId());
            return null;
        }
        handoverStaff = new TabHandoverStaff();
        TabHaobanClerkMainStoreRelated clerkMainStoreRelated = clerkMainStoreRelatedMapper.getLastNoStatusByStaffId(wxEnterpriseId, haobanStaff.getStaffId());
        if (null == clerkMainStoreRelated) {
            logger.info("没有主门店：{}", haobanStaff.getStaffId());
            //todo 需要过滤
//            return null;
        } else {
            handoverStaff.setStoreId(clerkMainStoreRelated.getStoreId());
        }
        handoverStaff.setStaffId(haobanStaff.getStaffId());
        handoverStaff.setHandoverUserId(wxUserId);
        handoverStaff.setWxEnterpriseId(wxEnterpriseId);
        handoverStaff.setHandoverStaffId(ToolUtil.randomUUID());
        handoverStaff.setCreateTime(new Date());
        handoverStaff.setUpdateTime(new Date());
        handoverStaff.setStatusFlag(1);
        handoverStaff.setStaffId(haobanStaff.getStaffId());
        return handoverStaff;
    }

    /**
     * 获取外部联系人
     *
     * @param infoDTO
     * @return
     */
    private TabHandoverExternal getHandExternal(QywxUnassignedInfoDTO infoDTO, TabHaobanWxEnterprise wxEnterprise) {
        TabHandoverExternal mid = new TabHandoverExternal();
        mid.setExternalUserId(infoDTO.getExternalUserid());
        mid.setWxEnterpriseId(wxEnterprise.getWxEnterpriseId());
        mid.setHandoverExternalId(ToolUtil.randomUUID());
        mid.setHaobanFriendFlag(0);
        TabHaobanExternalClerkRelated externalClerkRelated = externalClerkRelatedMapper.getLastByExTernalUseridAndWxUserIdNoStatus(wxEnterprise.getWxEnterpriseId(), infoDTO.getExternalUserid(), infoDTO.getHandoverUserid());
        if (null == externalClerkRelated) {
            mid.setExternalUserName("企业微信客户");
            mid.setRelationId("-1");
        } else {
            mid.setRelationId(externalClerkRelated.getExternalClerkRelatedId());
            mid.setExternalUserName(externalClerkRelated.getExternalName());
            mid.setHaobanFriendFlag(1);
            mid.setExternalHeadUrl(externalClerkRelated.getHeadUrl());
        }
        return mid;
    }

    @Override
    public List<TabHandoverStaff> listByStoreId(String wxEnterpriseId, String storeId) {
        List<TabHandoverStaff> handoverStaffs = handoverStaffMapper.listByStoreId(wxEnterpriseId, storeId);
        if (CollectionUtils.isEmpty(handoverStaffs)) {
            return null;
        }
        return handoverStaffs;
    }

    @Override
    public Page<HandoverExternalDTO> pageByStaffIds(String wxEnterpriseId, String search, List<String> staffIds, List<String> transferIds, BasePageInfo pageInfo) {
        PageHelper.startPage(pageInfo);
        List<TabHandoverExternal> externalList = handoverExternalMapper.listByStaffIds(wxEnterpriseId, transferIds, staffIds, search);
        Page<HandoverExternalDTO> retPage = PageUtil.changePageHelperToCurrentPage(new PageInfo<>(externalList), HandoverExternalDTO.class);
        return retPage;
    }

    @Override
    public List<TabHandoverExternal> listByIds(String wxEnterpriseId, List<String> ids) {
        return handoverExternalMapper.listByIds(wxEnterpriseId, ids);
    }

    @Override
    public String saveHandoverTransfer(TabHandoverTransfer transfer, List<String> handoverExternalIds) {
        TabHandoverTransfer tabHandoverTransfer = handoverTransferMapper.getByHandoverStaffIdAndTakeoverStaffId(transfer.getWxEnterpriseId(), transfer.getHandoverStaffId(), transfer.getTakeoverStaffId());
        String transferId = null;
        if (null != tabHandoverTransfer) {
            handoverTransferMapper.addCount(tabHandoverTransfer.getHandoverTransferId(), transfer.getExternalCount());
            transferId = tabHandoverTransfer.getHandoverTransferId();
        } else {
            transfer.setHandoverTransferId(ToolUtil.randomUUID());
            transfer.setStatusFlag(1);
            transfer.setCreateTime(new Date());
            transfer.setUpdateTime(new Date());
            handoverTransferMapper.insertSelective(transfer);
            transferId = transfer.getHandoverTransferId();
        }
        //分配初始化
        handoverExternalMapper.updateAfterTransfer(handoverExternalIds, transferId);
        return transferId;
    }

    @Override
    public List<TabHandoverTransfer> listTransferByStoreId(String wxEnterpriseId, String storeId) {
        List<TabHandoverTransfer> handoverTransfers = handoverTransferMapper.listTransferByStoreId(wxEnterpriseId, storeId);
        return handoverTransfers;
    }

    @Override
    public List<TabHandoverTransfer> listTransferByHandoverStaffId(String wxEnterpriseId, String handoverStaffId) {
        return handoverTransferMapper.listTransferByHandoverStaffId(wxEnterpriseId, handoverStaffId);
    }

    @Override
    public List<TabHandoverTransfer> listTransferByTakeoverStaffId(String wxEnterpriseId, String takeoverStaffId) {
        return handoverTransferMapper.listTransferByTakeoverStaffId(wxEnterpriseId, takeoverStaffId);
    }
}
