package com.gic.haoban.manage.service.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.ToolUtil;
import com.gic.haoban.manage.api.dto.SecretSettingDTO;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.service.dao.mapper.SecretSettingMapper;
import com.gic.haoban.manage.service.entity.TabSecretSetting;
import com.gic.haoban.manage.service.service.SecretSettingService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * Created 2020/6/3.
 *
 * @author hua
 */
@Service
public class SecretSettingServiceImpl implements SecretSettingService {

    private static final Logger logger= LoggerFactory.getLogger(SecretSettingServiceImpl.class);

    @Autowired
    private SecretSettingMapper secretSettingMapper;

    @Override
    public boolean saveSecretSetting(SecretSettingDTO secretSetting) {
        if (secretSetting.getSecretId()!=null) {
            secretSetting.setUpdateTime(new Date());
            secretSettingMapper.updateByPrimaryKeySelective(EntityUtil.changeEntityByJSON(TabSecretSetting.class,secretSetting));
            return true;
        }else{
            secretSetting.setSecretId(ToolUtil.randomUUID());
            secretSetting.setCreateTime(new Date());
            secretSetting.setUpdateTime(new Date());
            secretSetting.setStatusFlag(1);
            secretSettingMapper.insert(EntityUtil.changeEntityByJSON(TabSecretSetting.class,secretSetting));
            return true;
        }
    }

    @Override
    public boolean delSecretSetting(String secretId) {
        TabSecretSetting tabSecretSetting = new TabSecretSetting();
        tabSecretSetting.setSecretId(secretId);
        tabSecretSetting.setStatusFlag(0);
        tabSecretSetting.setUpdateTime(new Date());
        int i = secretSettingMapper.updateByPrimaryKeySelective(tabSecretSetting);
        return i>0;
    }

    @Override
    public List<SecretSettingDTO> listSecretSetting(String wxEnterpriseId) {
        List<TabSecretSetting> list = secretSettingMapper.listSecretSetting(wxEnterpriseId);
        return EntityUtil.changeEntityListByJSON(SecretSettingDTO.class,list);
    }

    @Override
    public SecretSettingDTO getSecretSetting(String wxEnterpriseId, int secretType) {
        if (secretType == SecretTypeEnum.MEMBER_WAPP.getVal()) {
            logger.info("会员小程序配置使用getMemberSecretSetting接口");
            return null;
        }
        TabSecretSetting secretSetting = secretSettingMapper.getSecretSetting(wxEnterpriseId, secretType, null);
        return EntityUtil.changeEntityByJSON(SecretSettingDTO.class,secretSetting);
    }

    @Override
    public SecretSettingDTO getMemberSecretSetting(String wxEnterpriseId, String enterpriseId) {
        TabSecretSetting secretSetting = secretSettingMapper.getSecretSetting(wxEnterpriseId, SecretTypeEnum.MEMBER_WAPP.getVal(), enterpriseId);
        return EntityUtil.changeEntityByJSON(SecretSettingDTO.class,secretSetting);
    }
}
