package com.gic.haoban.manage.service.service.out.impl;

import java.io.InputStream;
import java.net.URL;
import java.util.*;
import java.util.stream.Collectors;

import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.service.entity.TabMiniprogramSetting;
import com.gic.haoban.manage.service.service.MiniprogramSettingService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.JSONResponse;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.GlobalVar;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.MaterialCategoryDTO;
import com.gic.haoban.manage.api.dto.MaterialDTO;
import com.gic.haoban.manage.api.dto.WxEnterpriseDTO;
import com.gic.haoban.manage.api.service.MaterialApiService;
import com.gic.haoban.manage.service.config.Config;
import com.gic.haoban.manage.service.entity.TabHaobanMaterial;
import com.gic.haoban.manage.service.service.MaterialCategoryService;
import com.gic.haoban.manage.service.service.MaterialService;
import com.gic.haoban.manage.service.service.WxEnterpriseService;
import com.gic.thirdparty.api.dto.PicUploadResDTO;
import com.gic.wechat.api.enums.QywxMediaTypeEnum;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.github.pagehelper.PageHelper;

@Service
public class MaterialApiServiceImpl implements MaterialApiService {
	
	@Autowired
	private MaterialCategoryService materialCategoryService;
	
	@Autowired
	private MaterialService materialService;
	
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService;
	
	@Autowired
	private WxEnterpriseService wxEnterpriseService;
	
	@Autowired
	private Config config;

    @Autowired
    private MiniprogramSettingService miniprogramSettingService;

    private static Logger logger  = LoggerFactory.getLogger(MaterialApiServiceImpl.class);

	@Override
	public boolean hasCategoryNameExsit(String categoryName, String categoryParentId,String categoryId) {
		MaterialCategoryDTO category = materialCategoryService.hasCategoryNameExsit(categoryName,categoryParentId);
		if(category != null){
			if(category.getCategoryId().equals(categoryId)){
				return false;
			}
			else{
				return true;
			}
		}
		return false;
		
	}

	@Override
	public void insertCategory(MaterialCategoryDTO materialCategoryDTO) {
		materialCategoryService.insert(materialCategoryDTO);
		
	}

	@Override
	public List<MaterialCategoryDTO> listCategory(String wxEnterpriseId) {
		return materialCategoryService.listCategory(wxEnterpriseId);
	}

	@Override
	public MaterialCategoryDTO selectMaterialCategoryById(String categoryId) {
		return materialCategoryService.selectMaterialCategoryById(categoryId);
	}

	@Override
	public void editCategory(MaterialCategoryDTO materialCategoryDTO) {
		materialCategoryService.editCategory(materialCategoryDTO);

		
	}

	@Override
	public List<MaterialDTO> listMaterialByCategoryId(String categoryId) {
		return materialService.listMaterialByCategoryId(categoryId);
	}

	@Override
	public void insertMaterial(MaterialDTO materialDTO) {
		Integer type = materialDTO.getMaterialType();
		String wxEnterpriseId = materialDTO.getWxEnterpriseId();
		WxEnterpriseDTO enterprise = wxEnterpriseService.selectById(wxEnterpriseId);
		if(type == null){
			return;
		}
		if(enterprise == null){
			return;
		}
		QywxMediaTypeEnum fileType = null;
		String url = "";
		if(type == 2){
			fileType = QywxMediaTypeEnum.IMAGE;
			url = materialDTO.getImgUrl();
		}
		if(type == 4){
			fileType = QywxMediaTypeEnum.VIDEO;
			url = materialDTO.getLink();
		}
		if(type == 5){
			fileType = QywxMediaTypeEnum.FILE;
			url = materialDTO.getLink();
		}
        if (type == 6) {
            fileType = QywxMediaTypeEnum.IMAGE;
            url = materialDTO.getImgUrl();
        }
        if(type == 3){
			String imgUrl = materialDTO.getImgUrl();
			logger.info("【上传图片】imgUrl={}",imgUrl);
			JSONResponse response = qywxSuiteApiService.uploadImage(enterprise.getCorpid(), config.getWxSuiteid(), getFileByte(imgUrl));
			logger.info("【上传图片返回】response={}",JSON.toJSONString(response));
			String wxImgUrl = response.getResult() == null ? "" : response.getResult().toString();
			materialDTO.setWxImgUrl(wxImgUrl);
		}
		if(fileType != null){
			String[] arr = url.split("/");
			int count = arr.length;
			JSONResponse jp = qywxSuiteApiService.uploadMedia(enterprise.getCorpid(), config.getWxSuiteid(), getFileByte(url),arr[count - 1], fileType.getCode());
			if(jp.getErrorCode() == 0){
				materialDTO.setWxLastUploadTime(new Date());
				materialDTO.setMediaId(jp.getResult().toString());
				materialService.insertMaterial(materialDTO);
			}
		}else{
			materialService.insertMaterial(materialDTO);
		}
	}

	@Override
	public MaterialDTO selectMaterialById(String materialId) {
		return materialService.selectMaterialById(materialId);
	}

	@Override
    public List<MaterialDTO> listMaterialByIds(List<String> materialIds) {
        return materialService.listMaterialByIds(materialIds);
    }

    @Override
    public void editMaterial(MaterialDTO materialDTO) {
        Integer type = materialDTO.getMaterialType();
        String materialId = materialDTO.getMaterialId();
        MaterialDTO old = materialService.selectMaterialById(materialId);
        String wxEnterpriseId = old.getWxEnterpriseId();
        WxEnterpriseDTO enterprise = wxEnterpriseService.selectById(wxEnterpriseId);
        if(type == null){
            return;
        }
        if(enterprise == null){
            return;
        }
        QywxMediaTypeEnum fileType = null;
        String url = "";
        String oldUrl = "";
        if(type == 2){
            fileType = QywxMediaTypeEnum.IMAGE;
            url = materialDTO.getImgUrl();
            oldUrl = old.getImgUrl();
        }
        if(type == 4){
            fileType = QywxMediaTypeEnum.VIDEO;
            url = materialDTO.getLink();
            oldUrl = old.getLink();
        }
        if(type == 5){
            fileType = QywxMediaTypeEnum.FILE;
            url = materialDTO.getLink();
            oldUrl = old.getLink();
        }
        if(type == 3){
            String imgUrl = materialDTO.getImgUrl();
            String oldImgUrl = materialDTO.getImgUrl();
            if(!imgUrl.equals(oldImgUrl)){
                JSONResponse response = qywxSuiteApiService.uploadImage(enterprise.getCorpid(), config.getWxSuiteid(), getFileByte(imgUrl));
                String wxImgUrl = response.getResult() == null ? "" : response.getResult().toString();
                materialDTO.setWxImgUrl(wxImgUrl);
            }
        }
        if(fileType != null && !url.equals(oldUrl)){
            String[] arr = url.split("/");
            int count = arr.length;
            JSONResponse jp = qywxSuiteApiService.uploadMedia(enterprise.getCorpid(), config.getWxSuiteid(), getFileByte(url),arr[count - 1], fileType.getCode());
            if(jp.getErrorCode() == 0){
                materialDTO.setWxLastUploadTime(new Date());
                materialDTO.setMediaId(jp.getResult().toString());
                materialService.edit(materialDTO);
            }
        }else{
            materialService.edit(materialDTO);
        }

    }
	
	@Override
	public String  reUpdalodMetail(String materialId){
		MaterialDTO old = materialService.selectMaterialById(materialId);
		Integer type = old.getMaterialType();
		if(type == null){
			return "";
		}
		String wxEnterpriseId = old.getWxEnterpriseId();
		WxEnterpriseDTO enterprise = wxEnterpriseService.selectById(wxEnterpriseId);
		if(enterprise == null){
			return "";
		}
		String url = "";
		String oldUrl = "";
		QywxMediaTypeEnum fileType = null;
		if(type == 2){
			fileType = QywxMediaTypeEnum.IMAGE;
			url = old.getImgUrl();
		}
		if(type == 4){
			fileType = QywxMediaTypeEnum.VIDEO;
			url = old.getLink();
		}
		if(type == 5){
			fileType = QywxMediaTypeEnum.FILE;
			url = old.getLink();
        } else if (type == 6) {
            fileType = QywxMediaTypeEnum.IMAGE;
            url = old.getImgUrl();
        }
        String[] arr = url.split("/");
		int count = arr.length;
		JSONResponse jp = qywxSuiteApiService.uploadMedia(enterprise.getCorpid(), config.getWxSuiteid(), getFileByte(url),arr[count - 1], fileType.getCode());
		if(jp.getErrorCode() == 0){
			old.setWxLastUploadTime(new Date());
			old.setMediaId(jp.getResult().toString());
			materialService.edit(old);
		}
		return old.getMediaId();
	}

	private List<String> listSubCategoryIdsByParentId(String categoryId) {
		List<String> categoryIds = new ArrayList<>();
		categoryIds.add(categoryId);
		if (StringUtils.isBlank(categoryId)) {
			return categoryIds;
		}
		List<MaterialCategoryDTO> categoryDTOList = materialCategoryService.listByParentCategory(categoryId);
		if (CollectionUtils.isEmpty(categoryDTOList)) {
			return categoryIds;
		}
		for (MaterialCategoryDTO categoryDTO : categoryDTOList) {
			categoryIds.addAll(this.listSubCategoryIdsByParentId(categoryDTO.getCategoryId()));
		}
		return categoryIds;
	}

	@Override
	public Page<MaterialDTO> listMaterial(String wxEnterpriseId, String keyword, String categoryId,
			Integer materialType,BasePageInfo pageInfo) {
		List<String> categoryIds = this.listSubCategoryIdsByParentId(categoryId);
		PageHelper.startPage(pageInfo.getPageNum(), pageInfo.getPageSize());
		com.github.pagehelper.Page<TabHaobanMaterial> page = materialService.listMaterial(wxEnterpriseId, keyword, categoryIds, materialType);
		List<TabHaobanMaterial> result = page.getResult();
        List<TabMiniprogramSetting> settings = miniprogramSettingService.listMiniprogram(wxEnterpriseId);
        if (CollectionUtils.isNotEmpty(result) && CollectionUtils.isNotEmpty(settings)) {
            Map<String, TabMiniprogramSetting> settingMap = settings.stream().collect(Collectors.toMap(dto -> dto.getAppId(), dto -> dto));
            result.forEach(dto -> {
                TabMiniprogramSetting miniprogramSetting = settingMap.get(dto.getAppId());
                dto.setMaterialDesc(miniprogramSetting == null ? "--" : miniprogramSetting.getMiniprogramName());
            });
        }
        return  PageUtil.changePageHelperToCurrentPage(page,MaterialDTO.class);
	}
	
	public static void main(String[] args) {
		String str = "https://other-1251519181.cos.ap-shanghai.myqcloud.com/haoban/20200518101933/【④好办-数据】(1)(1).xlsx";
		int count = str.split("/").length;
		System.err.println(str.split("/")[count - 1]);
	}
	
	private static byte[] getFileByte(String url) {

		try {
			InputStream in = new URL(url).openStream();
			byte[] data = IOUtils.toByteArray(in);
			return data;
		}catch (Exception e){
			logger.info("【异常】"+e.getMessage(),e);
		}
		return null;
	}

	@Override
	public List<MaterialCategoryDTO> listByParentCategory(String categoryId) {
		return materialCategoryService.listByParentCategory(categoryId);
	}

	@Override
	public ServiceResponse<Void> shareMaterial(String wxEnterpriseId, String toWxEnterpriseId, String materialId) {
		ServiceResponse<Void> res = new ServiceResponse<>();
		MaterialDTO materialDTO = materialService.selectMaterialById(materialId);
		if (null == materialDTO) {
			logger.info("不存在该素材");
			res.setCode(2);
			res.setMessage("不存在该素材");
			return res;
		}
		if (!materialDTO.getFromMaterialId().equals("-1")) {
			logger.info("该素材不允许分享");
			res.setCode(3);
			res.setMessage("该素材不允许分享");
			return res;
		}
		MaterialDTO fromMaterialDTO = materialService.getMaterialByFromMaterialId(materialId, toWxEnterpriseId);
		if (fromMaterialDTO != null) {
			logger.info("已经存在素材，不能重复分享");
			res.setCode(4);
			res.setMessage("已经存在素材，不能重复分享");
			return res;
		}
		materialDTO.setWxEnterpriseId(toWxEnterpriseId);
		materialDTO.setFromMaterialId(materialId);
		materialDTO.setMediaId(null);
		materialDTO.setWxLastUploadTime(null);
		materialService.insertMaterial(materialDTO);
		return res;
	}

	@Override
	public List<MaterialDTO> listByFromMaterialId(String materialId) {
		List<MaterialDTO> materialDTOS = materialService.listMaterialByFromMateralId(materialId);
		return materialDTOS;
	}

	@Override
	public void delMaterial(List<String> ids) {
		materialService.delmaterialByIds(ids);
	}

    @Override
    public MaterialDTO getHasChangeMadieMaterialById(String materialId) {
        MaterialDTO materialDTO = this.selectMaterialById(materialId);
        if (null == materialDTO) {
            return null;
        }
        Date lastUploadTime = materialDTO.getWxLastUploadTime();
        Date date = new Date();
        Date compareDate = DateUtils.addDays(date, -3);
        if (lastUploadTime != null && lastUploadTime.after(compareDate) && StringUtils.isNotBlank(materialDTO.getMediaId())) {
            return materialDTO;
        }
        Integer materialType = materialDTO.getMaterialType();
        String madiaId = materialDTO.getMediaId();
        //需要上传的类型
        List<Integer> typeList = Arrays.asList(2, 4, 5, 6);
        if (typeList.contains(materialType)) {
            madiaId = this.reUpdalodMetail(materialId);
        }
        materialDTO.setMediaId(madiaId);
        materialDTO.setWxLastUploadTime(date);
        return materialDTO;
    }
}
