package com.gic.haoban.manage.service.task;

import com.alibaba.fastjson.JSONObject;
import com.gic.clerk.api.dto.ClerkDTO;
import com.gic.clerk.api.service.ClerkService;
import com.gic.dubbo.entity.ProviderLocalTag;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.manage.api.dto.DealParamMqDTO;
import com.gic.haoban.manage.api.dto.GicClerkDTO;
import com.gic.haoban.manage.api.enums.PreDealStatusEnum;
import com.gic.haoban.manage.api.enums.PreDealTypeEnum;
import com.gic.haoban.manage.api.enums.SyncTaskStatusEnum;
import com.gic.haoban.manage.api.service.DealSyncOperationApiService;
import com.gic.haoban.manage.api.service.DepartmentApiService;
import com.gic.haoban.manage.api.service.StaffApiService;
import com.gic.haoban.manage.service.entity.TabHaobanPreDealLog;
import com.gic.haoban.manage.service.entity.TabHaobanSyncTask;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.PreDealService;
import com.gic.haoban.manage.service.service.SyncTaskService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * Created 2020/4/13.
 *
 * @author hua
 */
@Service
public class QywxClerkSyncOperation implements BaseSyncOperation {
    private static final Logger logger = LoggerFactory.getLogger(QywxClerkSyncOperation.class);

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private StoreGroupService storeGroupService;

    @Autowired
    private PreDealService preDealService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private SyncTaskService syncTaskService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ClerkService clerkService;

    @Autowired
    private StaffApiService staffApiService;

    @Autowired
    private DealSyncOperationApiService dealSyncOperationApiService;

    @Override
    public void dealSingleByMq(DealParamMqDTO dealParamMqDTO, TabHaobanPreDealLog dataPre) {
        logger.info("微信成员处理：{}", JSONObject.toJSONString(dealParamMqDTO));

        boolean dealFlag = true;
        String reason="";
        //处理clerk
        try {
            ServiceResponse response = staffApiService.getWxSaveNew(dealParamMqDTO.getData(), dataPre.getWxEnterpriseId());
            if (response.getCode()!=1) {
                dealFlag=false;
                reason=response.getMessage();
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.info("微信成员同步失败：{}", e);
            reason = "同步异常:";
            dealFlag = false;
        } finally {
            if (!dealFlag) {
                dealException(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), reason);
            } else {
                dealSuccess(dealParamMqDTO.getTaskId(), dealParamMqDTO.getData(), dataPre.getEnterpriseId(), dataPre.getWxEnterpriseId());
            }
        }


    }

    @Override
    public void dealException(String taskId, String dataId, String enterpriseId, String reason) {
        logger.info("微信成员处理失败：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId,PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.exception.getVal(), reason);
        if (!b) {
            logger.info("微信成员处理异常:t:{}, d:{}", taskId, dataId);
            return;
        }
        checkDepartmentTask(taskId);

    }

    @Override
    public void dealSuccess(String taskId, String dataId, String enterpriseId, String wxEnterpriseId) {
        logger.info("微信成员处理成功：t:{},d:{}", taskId, dataId);
        boolean b = preDealService.updateStatusByDataId(taskId, dataId,PreDealTypeEnum.clerk.getVal(), PreDealStatusEnum.computed.getVal(), "成功");
        if (!b) {
            logger.info("微信成功修改错误！{}", dataId);
            return;
        }
        checkDepartmentTask(taskId);
    }

    @Override
    public void checkDepartmentTask(String taskId) {
        boolean b = preDealService.checkTask(taskId, PreDealTypeEnum.clerk.getVal());
        if (!b) {
            return;
        }
        logger.info("微信同步成功:{}", taskId);
        String key = "haoban_sync_clerk_task_" + taskId;
        RedisUtil.lock(key, 3L);
        TabHaobanSyncTask syncTask = syncTaskService.getSyncTask(taskId);
        //同步成功 进入门店处理
        if (syncTask.getStatusFlag().equals(SyncTaskStatusEnum.clerk_sync.getVal())) {
            int errCount = preDealService.countByTaskId(taskId, -1, PreDealStatusEnum.exception.getVal());
            if (errCount > 0) {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.exception_compute.getVal());
                dealSyncOperationApiService.unlockTask(syncTask.getWxEnterpriseId());
            } else {
                syncTaskService.updateTaskStatus(taskId, SyncTaskStatusEnum.compute.getVal());
                dealSyncOperationApiService.unlockTask(syncTask.getWxEnterpriseId());
                dealSyncOperationApiService.cleanDiffrence(syncTask.getWxEnterpriseId(),taskId);
            }
        }
        RedisUtil.unlock(key);
    }


}
