package com.gic.haoban.manage.web.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.api.enums.SecretTypeEnum;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.anno.IgnoreLogin;
import com.gic.haoban.manage.web.auth.AuthRequestUtil;
import com.gic.haoban.manage.web.config.Config;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;
import com.gic.wechat.api.service.qywx.QywxUserApiService;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@RestController
public class ApplicationController extends WebBaseController{
	private static final Logger log = LogManager.getLogger(ApplicationController.class);
	@Autowired
	private ApplicationApiService applicationApiService;
	@Autowired
	private QywxSuiteApiService qywxSuiteApiService;
	@Autowired
	private WxApplicationApiService wxApplicationApiService;
	@Autowired
	private WxEnterpriseApiService wxEnterpriseApiService;
	@Autowired
	private Config config;
	@Autowired
	private DepartmentApiService departmentApiService;
	@Autowired
	private ApplicationSettingApiService applicationSettingApiService;
	@Autowired
	private DealSyncOperationApiService dealSyncOperationApiService;

	@Autowired
	private QywxUserApiService qywxUserApiService;
	
	@RequestMapping("application-list")
	public HaobanResponse applicationList() {
		LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		List<ApplicationDTO> list = applicationApiService.listApplicationByWxEnterpriseId(wxEnterpriseId);
		return resultResponse(HaoBanErrCode.ERR_1,list);
	}
	@IgnoreLogin
	@RequestMapping("application-open-close-list")
	public HaobanResponse applicationList(String wxEnterpriseId) {
		if(StringUtils.isAnyBlank(wxEnterpriseId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		Map<String,Object> map = new HashMap<>();
		List<ApplicationDTO> list = applicationApiService.listApplicationByWxEnterpriseId(wxEnterpriseId);
		List<String> applicationIds = list.stream().map(s->s.getApplicationId()).collect(Collectors.toList());
		List<ApplicationSettingDTO> result = applicationSettingApiService.listOpenByWxEnterpriseIdAndApplicationIds(wxEnterpriseId,applicationIds);
		//开启
		List<ApplicationDTO> openList = new ArrayList<ApplicationDTO>();
		List<String> openApplicationIds = new ArrayList<>();
		if(CollectionUtil.isNotEmpty(result)){
			openApplicationIds = result.stream().map(s->s.getApplicationId()).collect(Collectors.toList());
			openList = applicationApiService.listByApplications(openApplicationIds);
		}
		//关闭
		List<ApplicationDTO> closeList = new ArrayList<ApplicationDTO>();
		applicationIds.removeAll(openApplicationIds);
		if(CollectionUtil.isNotEmpty(applicationIds)){
			closeList = applicationApiService.listByApplications(applicationIds);
		}		
		Object o = RedisUtil.getCache("haobanWxEnterpriseIdAppOpen");
		String wxEnterpriseIds = o == null?"":o.toString();
		if(!wxEnterpriseIds.contains(wxEnterpriseId)){
			//不开启
			Iterator<ApplicationDTO> it = openList.iterator();
			while(it.hasNext()){
				ApplicationDTO s = it.next();
				if(s.getApplicationName().contains("内购券")){
					it.remove();
				}
			}
			Iterator<ApplicationDTO> it2 = closeList.iterator();
			while(it2.hasNext()){
				ApplicationDTO s = it2.next();
				if(s.getApplicationName().contains("内购券")){
					it2.remove();
				}
			}
		}
		map.put("open", openList);
		map.put("close", closeList);
		return resultResponse(HaoBanErrCode.ERR_1,map);
	}
	
	@RequestMapping("open-or-close")
	public HaobanResponse applicationList(String wxEnterpriseId, String applicationId, Integer openFlag) {
		if(StringUtils.isAnyBlank(wxEnterpriseId,applicationId)){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		if(openFlag == null){
			return resultResponse(HaoBanErrCode.ERR_2);
		}
		applicationSettingApiService.saveSetting(wxEnterpriseId, applicationId, openFlag);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	@RequestMapping("has-bind-contract")
	public HaobanResponse hasBindContract(){
		LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(enterpriseDTO != null){
			SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal(),null);
			if (null == secretSetting||secretSetting.getCheckFlag()==0) {
				log.info("没有配置secret：{}", JSONObject.toJSONString(enterpriseDTO));
				return resultResponse(HaoBanErrCode.ERR_1, true);
			}
			if( enterpriseDTO.getContactFlag() != null && enterpriseDTO.getContactFlag() == 0){
				String taskId = dealSyncOperationApiService.createQywxTask(wxEnterpriseId, "同步企业微信架构", login.getStaffDTO().getStaffId(), "首次同步企业微信架构");
				if(StringUtils.isBlank(taskId)){
					log.info("【是否绑定】taskId={}",taskId);
					return resultResponse(HaoBanErrCode.ERR_1, true);
				}
				//this.departmentApiService.initwxDepartmentMQ(enterpriseDTO.getWxEnterpriseId());
				dealSyncOperationApiService.dealQywxDepartment(taskId, wxEnterpriseId);
				enterpriseDTO.setContactFlag(1);
				wxEnterpriseApiService.update(enterpriseDTO);
				
				applicationApiService.insertApplicationRelation(wxEnterpriseId);
//				log.info("【初始化部门调用】{},{},{}",RouterConstant.INIT_WX_DEPARTMENT_SERVICENAME,RouterConstant.INIT_WX_DEPARTMENT_METHODNAME,wxEnterpriseId);
//		        try {
//					GICMQClientUtil.getClientInstance().sendCommonMessage(RouterConstant.ROUTERTYPE, wxEnterpriseId,
//					 		RouterConstant.INIT_WX_DEPARTMENT_SERVICENAME, RouterConstant.INIT_WX_DEPARTMENT_METHODNAME);
//				} catch (Exception e) {
//					log.info(e.getMessage(),e);
//				}
			}
		}
		
		return resultResponse(HaoBanErrCode.ERR_1, true);
	}
	
	@RequestMapping("init-wx-department")
	public HaobanResponse initWxDepartment(){
		LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(enterpriseDTO != null){
			SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal(),null);
			if (null == secretSetting||secretSetting.getCheckFlag()==0) {
				log.info("没有配置secret");
				return resultResponse(HaoBanErrCode.ERR_1, false);
			}
			if(StringUtils.isNotBlank(secretSetting.getSecretVal()) && enterpriseDTO.getContactFlag() != null){
				String taskId = dealSyncOperationApiService.createQywxTask(wxEnterpriseId, "后门同步企业微信架构", login.getStaffDTO().getStaffId(), "后门同步企业微信架构");
				if(StringUtils.isBlank(taskId)){
					return resultResponse(HaoBanErrCode.ERR_1, false);
				}
				dealSyncOperationApiService.dealQywxDepartment(taskId, wxEnterpriseId);
			}
		}
		
		return resultResponse(HaoBanErrCode.ERR_1, true);
		
	}

	@RequestMapping("get-auth-url")
	public HaobanResponse getAuthUrl(){
		String suiteId = config.getSuiteId();
		log.info("应用ID：{}", suiteId);
		String redictUrl = config.getHost() + "callback";
		String authorizationUrl = this.qywxSuiteApiService.getGrantAuthorizationUrl(suiteId, redictUrl);
		return resultResponse(HaoBanErrCode.ERR_1, authorizationUrl);
	}

	@RequestMapping("callback")
	public void callback(String auth_code, HttpServletResponse response){
		String suiteId = config.getSuiteId();
		String result = this.qywxSuiteApiService.corpAuth(suiteId, auth_code);
		log.info("企业微信回调结果：{}. {}", auth_code, result);
		QywxCorpInfoDTO dto = JSON.parseObject(result, QywxCorpInfoDTO.class);
		WxApplicationDTO applicationDTO = new WxApplicationDTO();
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getEnterpriseBycorpId(dto.getCorpid());
		if(enterpriseDTO != null){
			applicationDTO.setName(dto.getAgentName());
			applicationDTO.setWxEnterpriseId(enterpriseDTO.getWxEnterpriseId());
			applicationDTO.setAgentId(dto.getAgentid());
			applicationDTO.setApplicationType(1);
			applicationDTO.setAgentName(dto.getAgentName());
			applicationDTO.setSiteId(dto.getSuiteId());
			wxApplicationApiService.addSuite(applicationDTO);
			SecretSettingDTO secretSetting = wxEnterpriseApiService.getSecretSetting(enterpriseDTO.getWxEnterpriseId(), SecretTypeEnum.HAOBAN_HELP.getVal(), null);
			if (null == secretSetting||secretSetting.getCheckFlag()==0) {
				log.info("没有配置secret：{}", JSONObject.toJSONString(enterpriseDTO));
				return;
			}
			this.departmentApiService.initwxDepartment(dto.getCorpid(), secretSetting.getSecretVal(), enterpriseDTO.getWxEnterpriseId());
			enterpriseDTO.setContactFlag(1);
			wxEnterpriseApiService.update(enterpriseDTO);
		}
		String redirectUri = config.getHost() + "index";
		redirectUri = redirectUri.replace("haoban-manage3-web", "haoban-3/#");
		try {
			response.sendRedirect(redirectUri);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	

	@RequestMapping("cancal-suite")
	@IgnoreLogin
	public HaobanResponse cancelSuite(String corpId, String suiteId){
		this.wxApplicationApiService.cancelSuite(corpId, suiteId);
		return resultResponse(HaoBanErrCode.ERR_0, "s");
	}


	@RequestMapping("list-external-user")
	@IgnoreLogin
	public Object list(String corpid, String suiteid, String userid){
		return this.qywxUserApiService.listExternalUserid(corpid, suiteid, userid);
	}

	@RequestMapping("corp-external-user")
	@IgnoreLogin
	public Object corpuserid(String corpid, String suiteid){
		return this.qywxUserApiService.listCorpExternalUser(corpid, suiteid);
	}

	@RequestMapping("get-external-user-info")
	@IgnoreLogin
	public Object userInfo(String corpid, String suiteid, String userid){
		return resultResponse(HaoBanErrCode.ERR_0, this.qywxUserApiService.getExternalUseridInfo(corpid, suiteid, userid));
	}

    @RequestMapping("get-external-unionid")
    @IgnoreLogin
    public Object userInfoUninid(String corpid, String suiteid, String unionid) {
        return resultResponse(HaoBanErrCode.ERR_0, this.qywxUserApiService.getExternalUseridByUnionid(corpid, suiteid, unionid));
    }

	@RequestMapping("sync-qywx")
	@IgnoreLogin
	public Object syncQywx(){
		LoginDTO login = (LoginDTO) AuthRequestUtil.getLoginUser();
		String wxEnterpriseId = login.getWxEnterpriseId();
		WxEnterpriseDTO enterpriseDTO = this.wxEnterpriseApiService.getOne(wxEnterpriseId);
		if(enterpriseDTO != null) {
			String taskId = dealSyncOperationApiService.createQywxTask(wxEnterpriseId, "后门同步企业微信架构", login.getStaffDTO().getStaffId(), "后门同步企业微信架构");
			if (StringUtils.isBlank(taskId)) {
				return resultResponse(HaoBanErrCode.ERR_0, false);
			}
			String ret = dealSyncOperationApiService.dealQywxDepartment(taskId, wxEnterpriseId);
			if (null!=ret) {
				return resultResponse(HaoBanErrCode.ERR_0,ret,null,null);
			}
		}
		return resultResponse(HaoBanErrCode.ERR_1);
	}
	
	@RequestMapping("init-application-relation")
	@IgnoreLogin
	public HaobanResponse insertApplicationRelation(String wxEnterpriseId){
		applicationApiService.insertApplicationRelation(wxEnterpriseId);
		return resultResponse(HaoBanErrCode.ERR_1);
	}
}
