package com.gic.haoban.manage.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.common.utils.EntityUtil;
import com.gic.haoban.common.utils.HaobanResponse;
import com.gic.haoban.manage.api.dto.DepartmentDTO;
import com.gic.haoban.manage.api.dto.MaidianLogDTO;
import com.gic.haoban.manage.api.service.*;
import com.gic.haoban.manage.web.errCode.HaoBanErrCode;
import com.gic.haoban.manage.web.qo.QywxXcxSendMessageQo;
import com.gic.redis.data.util.GicRateLimiter;
import com.gic.redis.data.util.RedisUtil;
import com.gic.wechat.api.dto.qywx.QywxXcxSendMessageDTO;
import com.gic.wechat.api.service.qywx.QywxSuiteApiService;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * Created 2020/4/9.
 *
 * @author hua
 */
@RestController
@RequestMapping("/test")
public class TestController extends WebBaseController {


    private static final Logger logger= LoggerFactory.getLogger(TestController.class);

    @Autowired
    private QywxSuiteApiService qywxSuiteApiService;

    @Autowired
    private MemberUnionidRelatedApiService memberUnionidRelatedApiService;

    private final ExecutorService pools = Executors.newFixedThreadPool(500);

    @Autowired
    private TestApiService testApiService;

    @Autowired
    private DepartmentApiService departmentApiService;

    @Autowired
    private MaidianLogApiService maidianLogApiService;

    @Autowired
    private WxEnterpriseRelatedApiService wxEnterpriseRelatedApiService;

    @RequestMapping("/send-message-test")
    public HaobanResponse testSendMessage(QywxXcxSendMessageQo qo) {
        QywxXcxSendMessageDTO messageDTO = EntityUtil.changeEntityByOrika(QywxXcxSendMessageDTO.class, qo);
        ArrayList<String> list = new ArrayList<>();
        list.add(qo.getUserId());

        messageDTO.setUserIds(list);
        boolean b = qywxSuiteApiService.sendMessage("ww9ede832a84b7ae5f",
                "ww2c34dc56739bb105", messageDTO);
        if (b) {
            return resultResponse(HaoBanErrCode.ERR_1);
        } else {
            return resultResponse(HaoBanErrCode.ERR_0);
        }
    }

    @RequestMapping("/clean-out")
    public HaobanResponse cleanMessage(String cid) {
        memberUnionidRelatedApiService.cleanByCid(cid);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/test-dubbo")
    public HaobanResponse dubbo(@RequestParam(defaultValue = "2000") Integer n,@RequestParam(defaultValue = "100") String t) {
        int i=n;
        while (i-->0) {
            int finalI = i;
            pools.execute(new Runnable(){
                @Override
                public void run() {
                    long start = System.currentTimeMillis();
                    testApiService.checkThread(finalI +"",Long.valueOf(t));
                    logger.info("时间:{}",System.currentTimeMillis()-start);
                }
            });
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/del-department-batch")
    public HaobanResponse departmentDel(String departmentId) {
        String key="del-department-batch";
        Object cache = RedisUtil.getCache(key);
        if (null != cache) {
            return resultResponse(HaoBanErrCode.ERR_0);
        }
        RedisUtil.setCache(key,1,120L, TimeUnit.SECONDS);
        DepartmentDTO departmentDTO = departmentApiService.selectById(departmentId);
        if (null == departmentDTO) {
            return resultResponse(HaoBanErrCode.ERR_0);
        }
        List<DepartmentDTO> departmentDTOS = departmentApiService.listByChainId(departmentDTO.getChainId(), departmentDTO.getWxEnterpriseId());
        TreeMap<Integer, List<DepartmentDTO>> treeMap = departmentDTOS.stream().collect(Collectors.groupingBy(DepartmentDTO::getLevel, TreeMap::new, Collectors.toList()));
        treeMap.descendingMap().forEach((midKey, val) -> {
            val.forEach(departmentDTO1 -> {
                departmentApiService.del(departmentDTO1.getDepartmentId());
            });
        });

        RedisUtil.delCache(key);
        return resultResponse(HaoBanErrCode.ERR_1);
    }


    @RequestMapping("/lock")
    public HaobanResponse lock(String name,Long time,int ty) {
        RedisUtil.lock(name, time, TimeUnit.SECONDS, 0L);
        if (ty==0) {
            if(RedisUtil.isLocked(name)){
                RedisUtil.unlock(name);
            }
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/maidian-log")
    public HaobanResponse maidian(String hbversion, BasePageInfo pageInfo) {
        Page<MaidianLogDTO> page = maidianLogApiService.queryList(hbversion, pageInfo);
        return resultResponse(HaoBanErrCode.ERR_1,page);
    }
    
    @RequestMapping("/deal-slef-qywx-sys-department")
    public HaobanResponse dealSys1(String corpid,String secretVal,String dataId) {
    	testApiService.listSelfQywxUser(corpid, secretVal, dataId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }
    
    @RequestMapping("/deal-all-qywx-sys-department")
    public HaobanResponse dealSys2(String corpid,String secretVal,String dataId) {
    	testApiService.listAllQywxUser(corpid, secretVal, dataId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/test-limiter")
    public HaobanResponse testLimiter(String key, String eid) {
        if (StringUtils.isAnyBlank(key,eid)) {
            return resultResponse(HaoBanErrCode.ERR_5);
        }
        GicRateLimiter instance = GicRateLimiter.getInstance("haoban-manage3-web");
        Long acquire = instance.acquire(key, eid);
        if (acquire == 0L) {
            return resultResponse(HaoBanErrCode.ERR_13);
        }
        return resultResponse(HaoBanErrCode.ERR_1);
    }

    @RequestMapping("/init-history")
    public HaobanResponse initHistory(String wxEnterpriseId) {
        wxEnterpriseRelatedApiService.initHistoryEnterprise(wxEnterpriseId);
        return resultResponse(HaoBanErrCode.ERR_1);
    }
}
